#include <cufft.h>
#include <unistd.h>

#define EXIT(x) exit(x);

#  define CUT_CHECK_ERROR(errorMessage) {                                    \
    cudaError_t err = cudaGetLastError();                                    \
    if( cudaSuccess != err) {                                                \
        fprintf(stderr, "Cuda error: %s in file '%s' in line %i : %s.\n",    \
                errorMessage, __FILE__, __LINE__, cudaGetErrorString( err) );\
        EXIT(EXIT_FAILURE);                                                  \
    }                                                                        \
    err = cudaThreadSynchronize();                                           \
    if( cudaSuccess != err) {                                                \
        fprintf(stderr, "Cuda error: %s in file '%s' in line %i : %s.\n",    \
                errorMessage, __FILE__, __LINE__, cudaGetErrorString( err) );\
        EXIT(EXIT_FAILURE);                                                  \
    }                                                                        \
    }


__global__ void dummykernel(float2 *idata, float2 *odata, int max )
{
	int idx = threadIdx.x + blockIdx.x * blockDim.x;

	if( idx < max  && idx>= 0) {
		odata[idx]= idata[idx];
	}
}







int main(int argc, char **argv) {
	if(argc>1) {
		cudaSetDevice(atoi(argv[1]));
	}

	int nx = 78;
	float2 *temp;
	cufftHandle rowplan;

 	cufftPlan1d(&(rowplan), nx, CUFFT_C2C, (nx/2) * nx  ); 
	
	// allocate much more than necessary
	cudaMalloc( (void**) &(temp), nx * nx *  nx * sizeof(float2) * 10);
	cudaMemset( temp, 0, sizeof(float2) * nx * nx * nx * 10);
	CUT_CHECK_ERROR("init");

	int max = nx *  nx * nx; 

	
while(1){

	dummykernel<<< max / 256 , 256  >>>( temp, temp, max );
	CUT_CHECK_ERROR("dummy kernel");

	cufftExecC2C( rowplan, (cufftComplex*) temp, (cufftComplex*) temp, CUFFT_FORWARD );

	CUT_CHECK_ERROR("fft");


}

	return 0;
}







